############################################################
###          DS ANALYSIS ENVIRONMENT: SET UP             ###
############################################################

# List of required packages
packages = c("DSI", 
             "DSOpal", 
             "dsBaseClient", 
             "ggplot2")

# Requirement exclusive of Windows Systems
if(Sys.info()["sysname"] == "Windows"){
  packages[length(packages)+1] = "Rtools"
}

# This loop checks whether or not the required packages are already installed or not, and only executes the installation procedure in negative case.
for(p in packages){
  
  print(sprintf("Checking package %s", p))
  
  # This condition check whether or not package 'p' is already installed.
  if(require(p, character.only = T) == FALSE){
    
    installation_repo <- "http://cran.us.r-project.org"
    
    # dsBaseClient package requires an specific repository to be installed from.
    if(p == "dsBaseClient"){
      
      installation_repo <- "http://cran.obiba.org"
    }
    
    install.packages(p, repos = installation_repo, dependencies = TRUE)
  }
  
  
  # Load packages (Rtools only requires installation, not loading).
  if(p != "Rtools"){
    
    library(p, character.only = T)
  }   
  
}


############################################################
############################################################
###           CONNECTING TO 1+ DATA SOURCES              ###
############################################################

# Variables to be set prior to any execution

urls <- c("https://uncover.itg.be")

project_names <- c("name of your project")

resource_names <- c("name of your resource")

users <- c("administrator")

passwords <- c("your password")


# Variable in which connections to each server will be stored.
builder <- DSI::newDSLoginBuilder()

url_ctr <- 0
# This loop will add each server-connection configuration to the "builder" variable.
for(i in 1:length(urls)){
  
  print(sprintf("Connecting to Server with URL: %s", urls[i]))
  
  builder$append(server = sprintf("study%s", url_ctr), url = urls[i],
                 user = users[i], password = passwords[i],
                 resource = sprintf("%s.%s", project_names[i], resource_names[i]),
                 driver = "OpalDriver", options="list(ssl_verifyhost=0,ssl_verifypeer=0)")
  
  url_ctr <- url_ctr+1
}

# Set up the connections previously defined and stored in "builder". Also, store data aggregation on server-side dataframe with name "myDFName".
logindata <- builder$build()
connections <- DSI::datashield.login(logins = logindata, assign = TRUE, symbol = "D")

datashield.assign.expr(connections, symbol = 'data', expr = quote(as.resource.data.frame(D)))


ds.colnames(x='data', datasources= connections)

ds.class('data$DMRAGEYR', datasources = connections)
############################################################
############################################################
###                       LOG-OUT                        ###
############################################################

# Always execute this command after closing your IDE to free server-side resources.
datashield.logout(connections) 

